/************************************************************************/
/* TaxSolve_uk_2003.c - OpenTaxSolver for UK Tax Return, for 2003.	*/
/* Copyright (C) 2002, 2003, 2004 - Aston Roberts			*/
/* 									*/
/* OTS Project Home Page and Updates:  					*/
/*		http://opentaxsolver.sourceforge.com/			*/
/* 									*/
/* Compile:   cc taxsolve_uk_2003.c -o taxsolve_uk_2003			*/
/* Run:       ./taxsolve_uk_2003 	 uk_2003.dat			*/
/* 									*/
/* This program follows the SA151c guide.				*/
/* 									*/
/* GNU Public License - GPL:						*/
/* This program is free software; you can redistribute it and/or	*/
/* modify it under the terms of the GNU General Public License as	*/
/* published by the Free Software Foundation; either version 2 of the	*/
/* License, or (at your option) any later version.			*/
/* 									*/
/* This program is distributed in the hope that it will be useful,	*/
/* but WITHOUT ANY WARRANTY; without even the implied warranty of	*/
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU	*/
/* General Public License for more details.				*/
/* 									*/
/* You should have received a copy of the GNU General Public License	*/
/* along with this program; if not, write to the Free Software		*/
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA		*/
/* 02111-1307 USA							*/
/* 									*/
/* Aston Roberts 1-3-2004	aston_roberts@yahoo.com			*/
/* Byron Williams 8-16-2004	(conversion to UK taxes)		*/
/************************************************************************/

#include <stdio.h>
#include <time.h>

#include "taxsolve_routines.c"
int status=0, age;
#define MAXSECT 25
#define MAXBOX 100
float C[MAXSECT][MAXBOX];	/* Values for SA151c form. */
float box[MAXSECT][MAXBOX];	/* Values for SA10x... forms. */
float boxA[MAXSECT][MAXBOX];	/* Values for alternates (Lloyd's, ministers, A/B) */

FILE *infile=0, *outf;

/* Get a box value. */
void getbox( char *boxname, float *value )
{
 char word[200];
 get_parameter( infile, 's', word, boxname);
 get_parameters( infile, 'f', value, boxname);
}

/* Get a box value, and print it. */
void GetBox( char *boxname, float *value )
{
 getbox( boxname, value );
 fprintf(outf, "%s = %6.2f\n", boxname, *value );
}


/* Show a C-box number and it's value, to output file. */
void showCboxf( int j, int k )
{ fprintf(outf, "C%d.%d = %6.2f\n", j, k, C[j][k]); }

/* Show a C-box number, it's value, and a message to output file. */
void showCboxfMsg( int j, int k, char *msg )
{ fprintf(outf, "C%d.%d = %6.2f 	%s\n", j, k, C[j][k], msg); }

/* Get a C box value. */
void get_C_box( int j, int k )
{
 char boxname[100], word[200];
 sprintf(boxname,"C%d.%d", j, k);
 getbox( boxname, &C[j][k] );
 showCboxf(j, k);
}

/* Get a C box value, but if less than zero, show zero. */
void get_C_box0( int j, int k )
{
 char boxname[100], word[200];
 sprintf(boxname,"C%d.%d", j, k);
 getbox( boxname, &C[j][k] );
 if (C[j][k] < 0.0) C[j][k] = 0.0;
 showCboxf(j, k);
}

void intialize_boxes()
{
 int j, k;
 for (j=0; j<MAXSECT; j++)
  for (k=0; k<MAXBOX; k++)
   {
    C[j][k] = 0.0;
    box[j][k] = 0.0;
    boxA[j][k] = 0.0;
   }
}

float smallerof( float a, float b ) { if (a<b) return a; else return b; }
float largerof( float a, float b ) { if (a>b) return a; else return b; }
float NotLessThanZero( float a )    { if (a<0.0) return 0.0; else return a; }

void Display_File( char *filename )
{
 FILE *infile;
 char line[500];

 infile = fopen(filename,"r");
 if (infile==0) {printf("Could not open %s\n", filename); return;}
 fgets(line, 500, infile);
 while (!feof(infile))
  {
   printf("%s", line);
   fgets(line, 500, infile);
  } 
 fclose(infile);
}


/*----------------------------------------------------------------------------*/

main (argc, argv)
int argc;
char *argv[];
{
 int i, j, k, itemize, year, month, day;
 char word[1000], outfile[1000], *tmpstr;
 time_t now;

 /* Decode any command-line arguments. */
 i = 1;  k=1;
 while (i < argc)
 {
  if (strcmp(argv[i],"-verbose")==0)  { verbose = 1; }
  else
  if (k==1)
   {
    infile = fopen(argv[i],"r");
    if (infile==0) {printf("ERROR: Parameter file '%s' could not be opened.\n", argv[i]); exit(1);}
    k = 2;
    /* Base name of output file on input file. */
    strcpy(outfile,argv[i]);
    tmpstr = strstr(outfile,".");
    if (tmpstr==0) strcat(outfile,".out"); else strcpy(tmpstr,".out");
   }
  else
   {printf("Unknown command-line parameter '%s'\n", argv[i]); exit(1);}
  i = i + 1;
 }

 if (infile==0) {printf("Error: No input file on command line.\n"); exit(1);}
 outf = fopen(outfile,"w");
 printf("Writing results to file:  %s\n", outfile);

 /* Pre-initialize all boxes to zeros. */
 intialize_boxes();


 /*-----------------------------------*/
 /* Get the data from the input file. */
 /*-----------------------------------*/

 /* Expect  UK IR return lines, something like:
	Title:  UK SA-100 1999 Return
	1.8    25,000	{ Total Income }
	1.9         0  { Commissions in 1.8 }
 */

 /* Accept Form's "Title" line, and put out with date-stamp for your records. */
 read_line( infile, word );
 now = time(0);
 fprintf(outf,"\n%s\n%s\n", word, ctime( &now ));
 fprintf(outf,"Summary of Input:\n");

  GetBox( "1.8",  &box[1][8] );	      /* Payments from P60 (or P45) */
  GetBox( "1.9",  &box[1][9] );       /* Payments not on P60, etc. - tips */
  GetBox( "1.10", &box[1][10] );      /* other payments (exclude expenses below, lump sums, compensation, benefits */
  GetBox( "1.11", &box[1][11] );      /* UK Tax deducted from P60/P45, tips, & other payments */
  GetBox( "1.12", &box[1][12] );      /* Assets transferred/payments made for you */
  GetBox( "1.13", &box[1][13] );      /* Vouchers, credit cardsand tokens */
  GetBox( "1.14", &box[1][14] );      /* Living accommodation */
  GetBox( "1.15", &box[1][15] );      /* Excess mileage allowances and passenger payments */
  GetBox( "1.16", &box[1][16] );      /* Company cars */
  GetBox( "1.17", &box[1][17] );      /* Fuel for company cars */
  GetBox( "1.18", &box[1][18] );      /* Vans */
  GetBox( "1.19", &box[1][19] );      /* Interest-free and low-interest loans */
  GetBox( "1.21", &box[1][21] );      /* Private medical or dental insurance */
  GetBox( "1.22", &box[1][22] );      /* Other benefits */
  GetBox( "1.23", &box[1][23] );      /* Expenses payments received and balancing charges */
  GetBox( "1.27", &box[1][27] );      /* Taxable lump sums, IR204 box B */
  GetBox( "1.28", &box[1][28] );      /* Taxable lump sums, IR204 box K */

  GetBox( "1.29", &box[1][29] );      /* Taxable Lump sums */
  GetBox( "1.30", &box[1][30] );      /* Tax deducted from payments in boxes 1.27 to 1.29, not in 1.11 */
  GetBox( "1.31", &box[1][31] );      /* Foreign earnings not taxable in the UK */
  GetBox( "1.32", &box[1][32] );      /* Travel and subsistence costs */
  GetBox( "1.33", &box[1][33] );      /* Fixed deductions for expenses */
  GetBox( "1.34", &box[1][34] );      /* Professional fees and subscriptions */
  GetBox( "1.35", &box[1][35] );      /* Other expenses and capital allowances incurred in doing your job */
  GetBox( "1.37", &box[1][37] );      /* Seafarers Earnings Deduction */
  GetBox( "1.38", &box[1][38] );      /* Foreign tax for which tax credit relief not claimed */

  GetBox( "1M.45", &boxA[1][45] );      /* Minister of religion Lump sums, IR204 box L */
  GetBox( "1M.50", &boxA[1][50] );      /* Ministers of Religion, Total tax paid */
  GetBox( "1M.51", &boxA[1][51] );      /* Ministers of Religion, Total taxable income */

  GetBox( "2.42", &box[2][42] );	/* Share Schemes income */

  GetBox( "3.85", &box[3][85] );	/* Loss offset against other income for 2003-04 */
  GetBox( "3.92", &box[3][92] );	/* Total taxable profits from business */
  GetBox( "3.97", &box[3][97] );	/* Deductions made by contractors on account of tax */
  GetBox( "3.98", &box[3][98] );	/* tax deducted from trading income */

  GetBox( "3L.12", &boxA[3][12] );	/* Total UK tax deducted from personal funds at Lloyd's */
  GetBox( "3L.50", &boxA[3][50] );	/* Tax deducted from dividends from overseas sources in Lloyd's funds  */
  GetBox( "3L.53", &boxA[3][53] );	/* Total Uk tax deducted, not in 3L.50 */
  GetBox( "3L.85", &boxA[3][85] );	/* Trading Loss offset against other 2003-04 income */
  GetBox( "3L.91", &boxA[3][91] );	/* Total taxable profits from membership of Lloyd's */

  GetBox( "4.15", &box[4][15] );	/* Partnership Loss offset against other income for 2003-04 */
  GetBox( "4.35", &box[4][35] );      	/* Untaxed income from foreign savings */
  GetBox( "4.57B", &boxA[4][57] );   	/* Share of partnership untaxed loss offset from total income */
  GetBox( "4.61", &box[4][61] );     	/* Allowable loss on furnished holiday lettings to sell off against other income */
  GetBox( "4.70", &box[4][70] );     	/* Share of partnership taxed income (liable at 20%) */
  GetBox( "4.70A", &boxA[4][70] );   	/* Share of partnership income taxed at 10% */
  GetBox( "4.73", &box[4][73] );     	/* Share of partnership profits (other than that liable at 20%) */
  GetBox( "4.74", &box[4][74] );	/* Share of Income Tax deducted from partnership income */
  GetBox( "4.75", &box[4][75] );	/* Share of CIS25 deductions */
  GetBox( "4.75A", &boxA[4][75] );	/* Share of tax deducted from trading income (not CIS25 deductions) */
  GetBox( "4.76", &box[4][76] );	/* Share of tax credits */

  GetBox( "5.16", &box[5][16] );        /* Furnished Holidat Lettings Loss offset against 2003-04 total income */
  GetBox( "5.21", &box[5][21] );        /* Tax deducted on Rents and other income from land and property */
  GetBox( "5.43", &box[5][43] );        /* Propoerty Profit for the year */ 
  GetBox( "5.44", &box[5][44] );        /* Other property Loss offset against total income */ 

  GetBox( "6.1",  &box[6][1] );      	/* Total chargeable foreign interest & income from overseas savings */
  GetBox( "6.1A", &boxA[6][1] );  	/* Uk tax on 6.1 */
  GetBox( "6.2",  &box[6][2] );   	/* Total chargeable foreign dividends from overseas */
  GetBox( "6.2A", &boxA[6][2] );  	/* Uk tax on 6.2 */
  GetBox( "6.3",  &box[6][3] );      	/* Total UK tax on Income from overseas land & property */
  GetBox( "6.3A", &boxA[6][3] );  	/* Dividend income from overseas plus 6.3 */
  GetBox( "6.4",  &box[6][4] );		/* Total chargeable Income from overseas land & property */
  GetBox( "6.4A", &boxA[6][4] ); 	/* Chargeabe dividend income from overseas plus 6.4 */
  GetBox( "6.5", &box[6][5] );      	/* Disposals of holdings in offshore funds */
  GetBox( "6.7", &box[6][7] );      	/* Tax paid on foreign life ins. policy gains */
  GetBox( "6.8", &box[6][8] );      	/* Gains on foreign life insurance policies */
  GetBox( "6.9", &box[6][9] );  	/* Foreign tax credit relief */

  GetBox( "7.2", &box[7][2] );      /* 'rate applicable to trusts', tax paid */
  GetBox( "7.3", &box[7][3] );      /* 'rate applicable to trusts', taxable amount */
  GetBox( "7.5", &box[7][5] );      /* the basic rate, tax paid */
  GetBox( "7.6", &box[7][6] );      /* the basic rate, taxable amount */
  GetBox( "7.8", &box[7][8] );      /* the lower rate, tax paid */
  GetBox( "7.9", &box[7][9] );      /* the lower rate, taxable amount */
  GetBox( "7.11", &box[7][11] );      /* the dividend rate, tax paid */
  GetBox( "7.12", &box[7][12] );      /* the dividend rate, taxable amount */
  GetBox( "7.14", &box[7][14] );      /* Tax paid on income from deceased persons, basic rate */
  GetBox( "7.15", &box[7][15] );      /* Taxable income from deceased persons, basic rate  */
  GetBox( "7.17", &box[7][17] );      /* Tax paid on income from deceased persons, lower rate*/
  GetBox( "7.18", &box[7][18] );      /* Taxable income from deceased persons, lower rate  */
  GetBox( "7.20", &box[7][20] );      /* Tax paid on income from deceased persons, repayable basic rate */
  GetBox( "7.21", &box[7][21] );      /* Taxable income from deceased persons, repayable basic rate */
  GetBox( "7.24", &box[7][24] );      /* Tax paid on income from deceased persons, non-repayable basic rate */
  GetBox( "7.27", &box[7][27] );      /* Taxable income from deceased persons, non-repayable basic rate */
  GetBox( "7.30", &box[7][30] );      /* Tax paid on income from deceased persons, non-repayable dividend rate */
  GetBox( "7.31", &box[7][31] );      /* Taxable income from deceased persons, non-repayable dividend rate */

  GetBox( "8.13A", &boxA[8][13] );      /* Capital loss against income */
  GetBox( "9.35", &box[9][35] );      /* Non-resident tax relief claim */

  GetBox( "10.1", &box[10][1] );      /* Taxable Untaxed Interest */
  GetBox( "10.3", &box[10][3] );      /* Tax deducted from bank interest */
  GetBox( "10.4", &box[10][4] );      /* Gross bank interest before tax */
  GetBox( "10.6", &box[10][6] );      /* Tax deducted from trusts interest */
  GetBox( "10.7", &box[10][7] );      /* Gross trust interest before tax */
  GetBox( "10.8", &box[10][8] );      /* Taxable National Savings interest */
  GetBox( "10.10", &box[10][10] );      /* Tax deducted from First National Bonds */
  GetBox( "10.11", &box[10][11] );      /* Gross National bond interest before tax */
  GetBox( "10.13", &box[10][13] );      /* Tax deducted from other UK savings */
  GetBox( "10.14", &box[10][14] );      /* Gross other UK saviungs before taxes */
  GetBox( "10.17", &box[10][17] );      /* Company Dividend/distributions plus credit */
  GetBox( "10.20", &box[10][20] );      /* Trust Dividend/distributions plus credit */
  GetBox( "10.23", &box[10][23] );      /* Scrip dividends from UK companies plus notional tax */
  GetBox( "10.26", &box[10][26] );      /* Non-qualifying distributions and loans written off, Taxable amount */
  GetBox( "11.1", &box[11][1] );      /* State Retirement Pension */
  GetBox( "11.2", &box[11][2] );      /* Widow's Pension or Bereavement Allowance */
  GetBox( "11.3", &box[11][3] );      /* Widowed Mother's Allowance or Widowed Parent's Allowance */
  GetBox( "11.4", &box[11][4] );      /* Industrial Death Benefit Pension */
  GetBox( "11.5", &box[11][5] );      /* Jobseeker's Allowance */
  GetBox( "11.6", &box[11][6] );      /* Invalid Care Allowance */
  GetBox( "11.7", &box[11][7] );      /* Statutory Sick, Maternity, Paternity Pay paid by IR */
  GetBox( "11.8", &box[11][8] );      /* Taxable Incapacity Benefit, deducted */
  GetBox( "11.9", &box[11][9] );      /* Taxable Incapacity Benefit, Gross amount before tax */
  GetBox( "11.11", &box[11][11] );      /* Pensions, Tax deducted */
  GetBox( "11.12", &box[11][12] );      /* Pensions, Gross amount before tax */
  GetBox( "11.13", &box[11][13] );      /* Deduction */
  GetBox( "12.5", &box[12][5] );      /* Gains on taxable UK life insurance policies */
  GetBox( "12.7", &box[12][7] );      /* Tax deducted on Gains on void life insurance policies in ISAs */
  GetBox( "12.9", &box[12][9] );      /* Corresponding deficiency relief */
  GetBox( "12.12", &box[12][12] );      /* Refunds of surplus funds from additional contributions, plus notional tax */
  GetBox( "13.2", &box[13][2] );      /* Tax deducted from Other taxable income */
  GetBox( "13.3", &box[13][3] );      /* Other taxable income, before tax */
  GetBox( "13.5", &box[13][5] );      /* Earlier years' losses used in 2002-03 */
  GetBox( "14.5", &box[14][5] );      /* Payments to retirement annuity contracts relief */
  GetBox( "14.9", &box[14][9] );      /* Payments to personal pension relief claimed */
  GetBox( "14.10", &box[14][10] );      /* Contributions to employer's schemes not deducted from pay */
  GetBox( "14.11", &box[14][11] );      /* Gross Free-Standing Additional Voluntary Contributions paid 02-03 */ 
  GetBox( "15.1", &box[15][1] );      /* Interest eligible for relief on qualifying loans */
  GetBox( "15.2", &box[15][2] );      /* Maintenance and alimony */
  GetBox( "15.3", &box[15][3] );      /* Venture Capital Trusts */
  GetBox( "15.4", &box[15][4] );      /* Enterprise Investment Schemes */
  GetBox( "15.7", &box[15][7] );      /* Community Investment Tax Relief */
  GetBox( "15.8", &box[15][8] );      /* Post-cessation expenses, ... */
  GetBox( "15.10", &box[15][10] );      /* Payments to trade union or friendly society for death benefits */
  GetBox( "15.11", &box[15][11] );      /* Widows and orphans relief */
  GetBox( "15A.5", &boxA[15][5] );      /* Gifts of qualifying investments to charities  real property */
  GetBox( "15A.6", &boxA[15][6] );  
  GetBox( "15A.7", &boxA[15][7] );
  GetBox( "16.17", &box[16][17] );       /* Married couple's surplus allowance */

  GetBox( "20.1", &box[20][1]);		/* Tax refunded in the year */


 get_C_box(2,18);	/* Age-related personal allowance */
 get_C_box(2,19);	/* Blind person's allowance */
 get_C_box(2,20);	/* Blind person's surplus allowance from spouse */
 get_C_box( 7, 45 );	/* If box 3L.25 is not zero or blank, goto page 32 and figure boxes 7.43 through 7.45, */
			/*   else enter 0. */
 get_C_box( 7, 48 );	/* Class 4 National Insurance contributions */
 get_C_box0( 7, 50 );	/* Underpaid tax for earlier years included in 2003-04 tax code */
 get_C_box0( 7, 51 );	/* Repayment of Student Loan */
 get_C_box0( 7, 54 );	/* 2003-04 Underpaid tax that you requested be included in 2004-05 tax code */
 get_C_box0( 8, 3 );	/* Increase in tax calculated by reference to earlier years (see page 20). Usually zero. */
 get_C_box0( 8, 5 );	/* Decrease in tax calculated by reference to earlier years (see page 20). Usually zero. */
 get_C_box0( 8, 6 );	/* If claiming carry-back (see page 20). Usually zero. */
 get_C_box0( 8, 7 );	/* Payments or credits made to your 2003-04 payments on account */
 get_C_box0( 8, 8 );	/* Any other payments or credits, not already included in boxes c8.5, c8.6, or c8.7, */
			/* made to your 2003-04 tax bill */
 get_C_box(14,24);	/* Capital Gains Tax */
			/*   If you have capital gains and box 8.8 on        */
			/*   your Capital Gains Pages is greater than zero,  */
			/*   go to page 30 to figure C14.24, else enter 0.   */
 get_C_box(15,28);	/* Special relief for qualifying distributions */
			/*   If you filled in box 15.12, go to page 33 to figure C16.6, else enter 0. */
 get_C_box(16,6);	/* Relief for tax paid on grant of option */
			/*   If you have an entry in box 2.76 on the Share Schemes Pages go to page 33 */
			/*   to figure box c17.15, else enter 0. */
 get_C_box(17,15);


 /*--------------------------*/
 /* Now do the calculations. */
 /*--------------------------*/
 fprintf(outf,"\nCalculations:\n");

 /* Stage 1 - Income. */

 C[1][1] = box[1][8] + box[1][9] + box[1][10];
 for (j=12; j<=23; j++) C[1][1] = C[1][1] + box[1][j];
 C[1][1] = C[1][1] + box[1][27] + box[1][28];
 for (j=31; j<=38; j++) C[1][1] = C[1][1] - box[1][j];
 C[1][1] = C[1][1] + (boxA[1][51] - boxA[1][45]);
 C[1][1] = NotLessThanZero( C[1][1] );
 showCboxf(1,1);	/* Employment Income */

 C[1][2] = NotLessThanZero( box[1][29] + boxA[1][45] );
 showCboxf(1,2);	/* Lump Sums */

 C[1][3] = NotLessThanZero( box[2][42] );	
 showCboxf(1,3);	/* Share Schemes income, from box 2.42 */

 C[1][4] = NotLessThanZero( box[3][92] );	
 showCboxf(1,4);	/* Self-employment income, from box 3.92 */

 C[1][5] = NotLessThanZero( boxA[3][91] );	
 showCboxf(1,5);	/* Lloyd's underwriters income, from box 3L.91 */

 C[1][5] = NotLessThanZero( box[4][73] );	
 showCboxf(1,6);	/* Partnerships (non-savings) income, box 4.73 */

 C[1][7] = NotLessThanZero( box[4][35] + box[4][70] );	
 showCboxf(1,7);	/* Partnerships (savings income), box 4.35 + 4.70 */

 C[1][9] = NotLessThanZero( box[5][43] );	
 showCboxf(1,9);	/* Land & property income, box 5.43 */

 C[1][10] = NotLessThanZero( box[6][4] + box[6][5] );	
 showCboxf(1,10);	/* Foreign income, boxes 6.4 + 6.5 */
 
 C[1][11] = NotLessThanZero( box[6][8] - (box[6][7] * 100.0/22.0) );	
 showCboxf(1,11);	/* Foreign gains, box 6.8 - box 6.7 x 100/22 */
  
 C[1][12] = NotLessThanZero( box[6][1] );	
 showCboxf(1,12);	/* Foreign savings income, box 6.1 */

 C[1][15] = NotLessThanZero( box[7][3] + box[7][6] + box[7][15] );	
 showCboxf(1,15);	/* Trust & Estates income, boxes 7.3 + 7.6 + 7.15 */
  
 C[1][16] = NotLessThanZero( box[7][9] + box[7][18] + box[7][27] );	
 showCboxf(1,16);	/* Trust & Estates savings income, boxes 7.9 + 7.18 + 7.27 */
  
 C[1][19] = NotLessThanZero( box[10][1] + box[10][4] + box[10][7] + box[10][8] + box[10][11] + box[10][14] );	
 showCboxf(1,19);	/* UK intereste savings imcome, 10.1 + 10.4 + 10.7 + 10.8 + 10.11 + 10.14 */

 for (j=1; j<=7; j++) C[1][21] = C[1][21] + box[11][j];
 C[1][21] = NotLessThanZero( C[1][21] + box[11][9] + (box[11][12] - box[11][13]) );	
 showCboxf(1,21);	/* UK pensions & benefits income, boxes 11.1 to 11.7, 11.9 + (11.12 minus 11.13) */
 
 C[1][22] = NotLessThanZero( box[12][2] + box[12][8] );	
 showCboxf(1,22);	/* Gains on life policies, etc., boxes 12.2 + 12.8 */

 C[1][25] = NotLessThanZero( box[13][3] - box[13][5] );	
 showCboxf(1,25);	/* Other income, box 13.3 minus box 13.5 */

 C[1][26] = C[1][1] + C[1][3] + C[1][4] + C[1][5] + C[1][6] + C[1][9] + C[1][10]
	  + C[1][15] + C[1][21] + C[1][25];
 showCboxf(1, 26);
 C[3][1] = C[1][26];

 C[1][27] = C[1][2] + C[1][11] + C[1][22];
 showCboxf(1, 27);
 C[3][2] = C[1][27];

 C[1][28] = C[1][7] + C[1][12] + C[1][16] + C[1][19];
 showCboxf(1, 28);
 C[3][3] = C[1][28];
			/* Categories to work out correct tax bill. */

 C[1][8] = NotLessThanZero( boxA[4][70] );	
 showCboxf(1,8);	/* C1.8, from box 4.70A */

 C[1][13] = NotLessThanZero( box[6][2] + boxA[6][4] );	
 showCboxf(1,13);	/* C1.13, from boxes 6.2 + 6.4A */
  
 C[1][14] = NotLessThanZero( box[6][7] * 100.0 / 22.0 );	
 showCboxf(1,14);	/* C1.14, from box 6.7 x 100/22 */

 C[1][17] = NotLessThanZero( box[7][12] + box[7][21] + box[7][30] );	
 showCboxf(1,17);	/* C1.17, from boxes 7.12 + 7.21 + 7.30 */
 
 C[1][18] = NotLessThanZero( box[7][24] );	
 showCboxf(1,18);	/* C1.18, from box 7.24 */

 C[1][20] = NotLessThanZero( box[10][17] + box[10][20] + box[10][23] + box[10][26] );	
 showCboxf(1,20);	/* C1.20, from boxes 10.17 + 10.20 + 10.23 + 10.26 */
 
 C[1][23] = NotLessThanZero( box[12][12] );	
 showCboxf(1,23);	/* C1.23, from box 12.12 */
  
 C[1][24] = NotLessThanZero( box[12][5] );	
 showCboxf(1,24);	/* C1.24, from box 12.5 */

 C[1][29] = C[1][8] + C[1][13] + C[1][17] + C[1][20];
 showCboxf(1, 29);
 C[3][4] = C[1][29];

 C[1][30] = C[1][18] + C[1][23];
 showCboxf(1, 30);
 C[3][5] = C[1][30];

 C[1][31] = C[1][14] + C[1][24];
 showCboxf(1, 31);
 C[3][6] = C[1][31];

 C[1][32] = C[1][26] + C[1][27] + C[1][28] + C[1][29] + C[1][30] + C[1][31];
 showCboxfMsg(1, 32, "Total Income");
 C[3][7] = C[1][32];

 /* Stage 2 - Deductions */
			/* Losses */
 C[2][1] = box[3][85] + boxA[3][85];	
 showCboxf(2,1);	/* Self-employment & Lloyd's Losses, boxes 3.85 + 3L.85 */
 C[2][2] = box[4][15] + boxA[4][57] + box[4][61];	
 showCboxf(2,2);	/* Partnership Losses, boxes 4.15 + 4.57B + 4.61 */
 C[2][3] = box[5][16] + box[5][44];	
 showCboxf(2,3);	/* Land & Property Losses, boxes 5.16 + 5.44 */
 C[2][4] = boxA[8][13];	
 showCboxf(2,4);	/* Capital Losses, box 8.13A*/
 C[2][5] = box[15][8];	
 showCboxf(2,5);	/* Post-cessation expenses, box 15.8 */
 C[2][6] = C[2][1] +  C[2][2] + C[2][3] + C[2][4] + C[2][5];
 showCboxf(2, 6);

 C[2][7] = box[7][31];	
 showCboxf(2,7);         /* Other deductions, boxes 7.31 */

 C[2][8] = C[1][15] + C[1][16] + C[1][17] + C[1][18];
 showCboxf(2, 8);

 C[2][9] = smallerof( C[2][7], C[2][8] );
 showCboxf(2, 9);

			/* Other deductions */
 C[2][10] = box[14][5] + box[14][10];	
 showCboxf(2,10);	/* Pension paymnets paid gross, boxes 14.5 + 14.10 */
 C[2][11] = box[15][1];	
 showCboxf(2,11);	/* Loan interest payments, box 15.1 */
 C[2][12] = box[15][9];	
 showCboxf(2,12);	/* Anuities * annual payments, box 15.9 */
 C[2][13] = 100.0 * C[2][12] / 78.0;
 showCboxf(2, 13);
 C[2][14] = boxA[15][6] + boxA[15][7];	
 showCboxf(2,14);	/* Gifts, boxes 15A.6 + 15A.7 */
 C[2][15] = box[15][10];	
 showCboxf(2,15);	/* Trade union/friendly society death bene, box 15.10 */
 C[2][16] = C[2][6] + C[2][9] + C[2][10] + C[2][11] + C[2][13] + C[2][14] + C[2][15];
 showCboxf(2, 16);

			/* Allowances */
 C[2][17] = 4615.0;	/* Personal allowance, */
 showCboxf(2, 16);
 showCboxf(2,18);	/* Age-related personal allowance */
 showCboxf(2,19);	/* Blind person's allowance */
 showCboxf(2,20);	/* Blind person's surplus allowance from spouse */
 C[2][21] = C[2][17] +  C[2][18] + C[2][19] + C[2][20];
 showCboxf(2, 21);
 C[2][22] = C[2][15] +  C[2][21];
 showCboxfMsg(2, 22, "Total deductions and allowances");
 C[3][11] = C[2][22];

 /* Stage 3 - TAKING AWAY DEDUCTIONS AND SOME PERSONAL ALLOWANCES FROM THE INCOME */
 C[3][1] = NotLessThanZero(C[3][1]);
 showCboxf(3, 1);
 C[3][2] = NotLessThanZero(C[3][2]);
 showCboxf(3, 2);
 C[3][3] = NotLessThanZero(C[3][3]);
 showCboxf(3, 3);
 C[3][4] = NotLessThanZero(C[3][4]);
 showCboxf(3, 4);
 C[3][8] = 1960.0;
 showCboxf(3, 8);
 C[3][9] = NotLessThanZero( C[3][1] - C[3][8] );
 showCboxf(3, 9);
 C[3][10] = NotLessThanZero( C[3][8] - C[3][1] );
 showCboxf(3, 10);
 showCboxf(3, 11);
 C[3][12] = NotLessThanZero( C[3][11] - C[3][9] );
 showCboxf(3, 12);
 C[3][13] = NotLessThanZero( C[3][9] - C[3][11] );
 showCboxf(3, 13);
 C[3][14] = NotLessThanZero( C[3][2] - C[3][12] );
 showCboxf(3, 14);
 C[3][15] = NotLessThanZero( C[3][12] - C[3][2] );
 showCboxf(3, 15);
 C[3][16] = NotLessThanZero( C[3][8] + C[3][13] );
 showCboxf(3, 16);
 C[3][17] = NotLessThanZero( C[3][10] + C[3][15] );
 showCboxf(3, 17);
 C[3][18] = NotLessThanZero( C[3][16] - C[3][17] );
 showCboxf(3, 18);
 C[3][19] = NotLessThanZero( C[3][1] + C[3][2] );
 showCboxf(3, 19);
 C[3][20] = NotLessThanZero( C[3][11] - C[3][19] );
 showCboxf(3, 20);
 C[3][21] = NotLessThanZero( C[3][3] - C[3][20] );
 showCboxf(3, 21);
 C[3][22] = NotLessThanZero( C[3][20] - C[3][3] );
 showCboxf(3, 22);
 C[3][23] = NotLessThanZero( C[3][4] - C[3][22] );
 showCboxf(3, 23);
 C[4][22] = C[3][23];
 C[7][30] = C[3][23];

 C[3][5] = NotLessThanZero(C[3][5]);
 showCboxf(3, 5);	/* Calculated in stage 1 above. */
 C[3][6] = NotLessThanZero(C[3][6]);
 showCboxf(3, 6);	/* Calculated in stage 1 above. */
 C[3][7] = NotLessThanZero(C[3][7]);
 showCboxf(3, 7);	/* Calculated in stage 1 above. */

 C[3][24] = NotLessThanZero( C[3][22] - C[3][4] );
 showCboxf(3, 24);
 C[3][25] = NotLessThanZero( C[3][5] - C[3][24] );
 showCboxf(3, 25);
 C[4][8] = NotLessThanZero( C[3][18] + C[3][25] );

 C[3][26] = NotLessThanZero( C[3][24] - C[3][5] );
 showCboxf(3, 26);
 C[3][27] = NotLessThanZero( C[3][6] - C[3][26] );
 showCboxf(3, 27);
 C[3][28] = NotLessThanZero( C[3][7] - C[3][11] );
 showCboxfMsg(3, 28, "Total taxable income");

 /* Stage 4: ALLOCATING YOUR DIFFERENT KINDS OF INCOME TO TAX BANDS - FIRST */

 C[4][1] = 28540.0;
 showCboxf(4, 1);

 C[4][2] = NotLessThanZero( box[12][9] );
 showCboxf(4,2);	/* Corresponding deficiency relief, from box 12.9 */
 C[4][3] = NotLessThanZero( box[14][9] + box[14][11] );
 showCboxf(4,3);	/* Pension payments (including FSAVCs), boxes 14.9 + 14.11 */
 C[4][4] = NotLessThanZero( boxA[15][5] );
 showCboxf(4,4);	/* Gift Aid, from box 15A.5 */

 C[4][5] = 100.0 * C[4][4] / 78.0;
 showCboxf(4, 5);
 C[4][6] = 1960.0;
 showCboxf(4, 6);
 C[4][7] = C[4][1] + C[4][2] + C[4][3] + C[4][5];
 showCboxf(4, 7);
 
 C[4][8] = NotLessThanZero( C[3][18] + C[3][25] );
 showCboxf(4, 8);
 C[4][13] = NotLessThanZero( C[4][6] - C[4][9] );
 showCboxf(4, 13);
 C[4][14] = NotLessThanZero( C[4][7] - C[4][11] );
 showCboxf(4, 14);
 C[4][15] = NotLessThanZero( C[3][21] );
 showCboxf(4, 15);
 C[4][20] = NotLessThanZero( C[4][13] - C[4][16] );
 showCboxf(4, 20);
 C[4][21] = NotLessThanZero( C[4][14] - C[4][18] );
 showCboxf(4, 21);
 C[4][22] = NotLessThanZero( C[3][23] );
 showCboxf(4, 22);
 C[4][27] = NotLessThanZero( C[4][20] - C[4][23] );	/* C[4][23] = ?? */
 showCboxf(4, 27);
 C[4][28] = NotLessThanZero( C[4][21] - C[4][25] );
 showCboxf(4, 28);
 C[4][29] = NotLessThanZero( C[3][14] );
 showCboxf(4, 29);
 C[4][34] = NotLessThanZero( C[4][27] - C[4][30] );	/* C[4][30] = ?? */
 showCboxf(4, 34);
 C[4][35] = NotLessThanZero( C[4][28] - C[4][32] );
 showCboxf(4, 35);
 C[4][36] = NotLessThanZero( C[3][27] );
 showCboxf(4, 36);
 C[7][12] = C[4][36];

	/* Page 11 */

 C[4][9] = smallerof( C[4][8], C[4][6] );
 showCboxf(4, 9);
 C[4][10] = NotLessThanZero( C[4][8] - C[4][9] );
 showCboxf(4, 10);
 C[4][11] = smallerof( C[4][10], C[4][7] );
 showCboxf(4, 11);
 C[4][12] = NotLessThanZero( C[4][10] - C[4][11] );
 showCboxf(4, 12);

 C[4][16] = smallerof( C[4][13], C[4][15] );
 showCboxf(4, 16);
 C[4][17] = NotLessThanZero( C[4][15] - C[4][16] );
 showCboxf(4, 17);
 C[4][18] = smallerof( C[4][14], C[4][17] );
 showCboxf(4, 18);
 C[4][19] = NotLessThanZero( C[4][17] - C[4][18] );
 showCboxf(4, 19);

 C[4][23] = smallerof( C[4][20], C[4][22] );
 showCboxf(4, 23);
 C[4][24] = NotLessThanZero( C[4][22] - C[4][23] );
 showCboxf(4, 24);
 C[4][25] = smallerof( C[4][21], C[4][24] );
 showCboxf(4, 25);
 C[4][26] = NotLessThanZero( C[4][24] - C[4][25] );
 showCboxf(4, 26);

 C[4][30] = smallerof( C[4][27], C[4][29] );
 showCboxf(4, 30);
 C[4][31] = NotLessThanZero( C[4][29] - C[4][30] );
 showCboxf(4, 31);
 C[4][32] = smallerof( C[4][28], C[4][31] );
 showCboxf(4, 32);
 C[4][33] = NotLessThanZero( C[4][31] - C[4][32] );
 showCboxf(4, 33);

 C[4][37] = smallerof( C[4][34], C[4][36] );
 showCboxf(4, 37);
 C[4][38] = NotLessThanZero( C[4][36] - C[4][37] );
 showCboxf(4, 38);
 C[4][39] = smallerof( C[4][35], C[4][38] );
 showCboxf(4, 39);
 C[4][40] = NotLessThanZero( C[4][38] - C[4][39] );
 showCboxf(4, 40);

	/* Stage 5: NOW WORK OUT THE INCOME TAX DUE ON EACH CATEGORY OF INCOME */
	/* Page 12 */

 /* Non-savings income, lump sum payments and gains on life policies */
 C[5][1] = C[4][9] + C[4][30];
 showCboxf(5, 1);
 C[5][2] = 0.1 * C[5][1];
 showCboxf(5, 2);
 C[5][3] = C[4][11] + C[4][32] + C[4][37] + C[4][39];
 showCboxf(5, 3);
 C[5][4] = 0.22 * C[5][3];
 showCboxf(5, 4);
 C[5][5] = C[4][12] + C[4][33] + C[4][40];
 showCboxf(5, 5);
 C[5][6] = 0.4 * C[5][5];
 showCboxf(5, 6);
 
 /* Savings income */
 C[5][7] = C[4][16];
 showCboxf(5, 7);
 C[5][8] = 0.1 * C[5][7];
 showCboxf(5, 8);
 C[5][9] = C[4][18];
 showCboxf(5, 9);
 C[5][10] = 0.20 * C[5][9];
 showCboxf(5, 10);
 C[5][11] = C[4][19];
 showCboxf(5, 11);
 C[5][12] = 0.4 * C[5][11];
 showCboxf(5, 12);
 
 /* Dividends, etc. */
 C[5][13] = C[4][23] + C[4][25];
 showCboxf(5, 13);
 C[5][14] = 0.1 * C[5][13];
 showCboxf(5, 14);
 C[5][15] = C[4][26];
 showCboxf(5, 15);
 C[5][16] = 0.329 * C[5][15];
 showCboxf(5, 16);
 C[5][17] = C[5][2] + C[5][4] + C[5][6] + C[5][8] + C[5][10] + C[5][12] + C[5][14] + C[5][16];
 showCboxf(5, 17);
 C[7][1] = C[5][17];

 /* STAGE 6: ADD UP TAX TAKEN OFF (EXCLUDING THE TAX CREDITS ATTACHED TO ANY DIVIDENDS) */

  C[6][1] = box[1][11] + box[1][30];
  showCboxf(6, 1);

  C[6][2] = boxA[1][50];
  showCboxf(6, 2);
						/* Trusts, etc. */
  C[6][8] = box[7][2] + box[7][5] + box[7][8] + box[7][14] + box[7][17] + box[7][20];
  showCboxf(6, 8);
						/* Self-employments */
  C[6][3] = box[3][97] + box[3][98];
  showCboxf(6, 3);
						/* Non-residence */
  C[6][9] = box[9][35];
  showCboxf(6, 9);
						/* Lloyd's Underwriters */
  C[6][4] = boxA[3][12] + boxA[3][50] + boxA[3][53];
  showCboxf(6, 4);
						/* UK savings & investments, excluding dividends */
  C[6][10] = box[10][3] + box[10][6] + box[10][10] + box[10][13];
  showCboxf(6, 10);
						/* Partnership */
  C[6][5] = box[4][74] + box[4][75] + boxA[4][75];
  showCboxf(6, 5);
						/* UK pensions */
  C[6][11] = box[11][8] + box[11][11];
  showCboxf(6, 11);
						/* Land and Property */
  C[6][6] = box[5][21];
  showCboxf(6, 6);
						/* Gains life policies, void ISAs */
  C[6][12] = box[12][7];
  showCboxf(6, 12);
						/* Foreign */
  C[6][7] = boxA[6][1] + boxA[6][2] + boxA[6][3];
  showCboxf(6, 7);
						/* Other Income */
  C[6][13] = box[13][2];
  showCboxf(6, 13);

  C[6][14] = 0.0;
  for (j=1; j<=13; j++)
   C[6][14] = C[6][14] + C[6][j];
  showCboxf(6, 14);			/* Total tax taken off. */
  C[7][53] = C[6][14];


 /* STAGE 7: TAX ADJUSTMENTS AND FURTHER ALLOWANCES - GETTING TO YOUR TAX BILL */

 C[7][1] = NotLessThanZero( C[5][17] );		/* Income Tax due */
 showCboxf(7, 1);

 if ((C[4][29] > 0.0) || (C[4][36] > 0.0))
  {
   printf("Box C7.2 calculation not handled. Workout pages 28+29.\n");
   do
    printf("Enter value for box C7.2 from C13.28 ? ");
   while (scanf("%f",&C[7][2]) != 1);
  }
 C[7][2] = NotLessThanZero( C[7][2] );
 showCboxf(7, 2);

 /* Venture Capital Trusts and Enterprise Investment Schemes */
 C[7][3] = NotLessThanZero( box[15][3] + box[15][4] );
 showCboxf(7, 3);

 C[7][4] = 0.20 * C[7][3];
 showCboxf(7, 4);

 /* Community Investment Tax Relief */
 C[7][5] = NotLessThanZero( box[15][7] );
 showCboxf(7, 5);

 C[7][6] = 0.05 * C[7][5];
 showCboxf(7, 6);

 /* Maintenance and alimony */
 C[7][7] = NotLessThanZero( box[15][2] );
 showCboxf(7, 7);

 /* Married couple's allowance */		/* <--- From C11.6, C11.11, C11.16 or C11.17 */
 C[7][8] = NotLessThanZero( C[7][8] );
 showCboxf(7, 8);

 /* Married couple's surplus allowance */
 C[7][9] = NotLessThanZero( box[16][17] );
 showCboxf(7, 9);
 
 C[7][10] = NotLessThanZero( C[7][7] + C[7][8] + C[7][9] );
 showCboxf(7, 10);

 C[7][11] = 0.10 * C[7][10];
 showCboxf(7, 11);

 C[7][12] = NotLessThanZero( C[4][36] );
 showCboxf(7, 12);
 
 C[7][13] = 0.22 * C[7][13];
 showCboxf(7, 13);

 /* Other notional tax */
 /* If you have an entry in box 10.23, 10.26, 12.12, 4.78, 7.23, 7.26, 3L.25 or 3L.39, */
 /* go to page 32 to work out the figure for box c15.28, else enter 0. */
 showCboxf(15,28);
 C[7][14] = NotLessThanZero( C[15][28] );
 showCboxf(7, 14);

 /* Foreign tax credit relief */
 C[7][15] = NotLessThanZero( box[6][9] );
 showCboxf(7, 15);
 
 /* Widows and orphans relief */
 if (box[15][11] > 22.0) C[7][16] = 22.0;
 else C[7][16] = NotLessThanZero( box[15][11] );
 showCboxf(7, 16);
 
 /* Special relief for qualifying distributions */
 /* If you filled in box 15.12, go to page 33 to figure C16.6, else enter 0. */
 showCboxf(16,6);
 C[7][17] = NotLessThanZero( C[16][6] );
 showCboxf(7, 17);

 /* Relief for tax paid on grant of option */
 /* If you have an entry in box 2.76 on the Share Schemes Pages go to page 33 */
 /* to figure box c17.15, else enter 0. */
 showCboxf(17,15);
 C[7][18] = NotLessThanZero( C[17][15] );
 showCboxf(7, 18);

 C[7][19] = C[7][2] + C[7][4] + C[7][6] + C[7][11] + C[7][13] + C[7][18];
 showCboxf(7, 19);

 C[7][20] = NotLessThanZero( C[7][1] - C[7][19] );
 showCboxfMsg(7, 20, "Income Tax due, after allowances and reliefs");

 /* 'Grossed up' Gift Aid payments */
 C[7][21] = NotLessThanZero( boxA[15][5] ) * 100.0/78.0;
 showCboxf(7, 21);

 C[7][22] = 0.22 * C[7][21];
 showCboxf(7, 22);
 
 /* Capital Gains Tax */
 /* If you have capital gains and box 8.8 on        */
 /* your Capital Gains Pages is greater than zero,  */
 /* go to page 30 to figure C14.24, else enter 0.   */
 showCboxf(14,24);
 C[7][23] = NotLessThanZero( C[14][24] );
 showCboxf(7, 23);
 C[7][57] = C[7][23];

 C[7][24] = NotLessThanZero( C[7][22] - C[7][23] );
 showCboxf(7, 24);

 C[7][25] = largerof( C[7][20], C[7][24] );
 showCboxfMsg(7, 25, "Income Tax due");

 C[7][26] = NotLessThanZero( C[18][7] );	/* <--- ?? */
 showCboxf(7, 26);

 C[7][27] = NotLessThanZero( C[7][25] + C[7][26] );	
 showCboxf(7, 27);

 /* Non-payable tax credits on dividends */
 C[7][28] = ((box[6][3] + box[4][76]) * 10.0) + box[7][12] + box[7][30] + box[10][17] + box[10][20];
 showCboxf(7, 28);

 C[7][29] = 0.10 * C[7][28];
 showCboxf(7, 29);

 C[7][30] = NotLessThanZero( C[3][23] );
 showCboxf(7, 30);

 C[7][31] = NotLessThanZero( C[7][28] - C[7][30] );
 showCboxf(7, 31);

 C[7][32] = NotLessThanZero( C[7][25] - C[7][20] );
 showCboxf(7, 32);

 C[7][33] = NotLessThanZero( C[7][13] - C[7][14] );
 showCboxf(7, 33);

 C[7][34] = smallerof( C[7][32], C[7][33] );
 showCboxf(7, 34);

 C[7][35] = 10.0 * C[7][34];
 showCboxf(7, 35);

 C[7][36] = NotLessThanZero( C[7][31] - C[7][35] );
 showCboxf(7, 36);

 C[7][37] = NotLessThanZero( C[7][24] - C[7][1] );
 showCboxf(7, 37);

 C[7][38] = 10.0 * C[7][37];
 showCboxf(7, 38);

 C[7][39] = NotLessThanZero( C[7][36] - C[7][38] );
 showCboxf(7, 39);

 C[7][40] = 0.10 * C[7][39];
 showCboxf(7, 40);

 C[7][41] = NotLessThanZero( C[7][29] - C[7][40] );
 showCboxf(7, 41);

 /* If box 3L.25 is not zero or blank, goto page 32 and figure boxes 7.43 through 7.45, */
 /* else enter 0. */
 showCboxf( 7, 45 );

 C[7][46] = NotLessThanZero( C[7][41] + C[7][45] );
 showCboxfMsg(7, 46, "Total tax credits on dividends");

 C[7][47] = NotLessThanZero( C[7][27] - C[7][46] );
 showCboxfMsg(7, 47, "Income Tax due after taking off tax credits");

 /* Class 4 National Insurance contributions */
 showCboxf( 7, 48 );
 
 C[7][49] = NotLessThanZero( C[7][47] + C[7][48] );
 showCboxf(7, 49);

 /* Underpaid tax for earlier years included in 2003-04 tax code */
 showCboxf( 7, 50 );

 /* Repayment of Student Loan */
 showCboxf( 7, 51 );
 
 C[7][52] = NotLessThanZero( C[7][49] + C[7][50] + C[7][51] );
 showCboxf(7, 52);

 C[7][53] = C[6][14];
 showCboxf(7, 53);

 /* 2003-04 Underpaid tax that you requested be included in 2004-05 tax code */
 showCboxf( 7, 54 );
 
 C[7][55] = C[7][53] + C[7][54];
 showCboxf(7, 55);

 C[7][56] = C[7][52] - C[7][55];
 showCboxf(7, 56);

 C[7][57] = C[7][23];
 showCboxf(7, 57);

 C[7][58] = C[7][56] + C[7][57];
 showCboxf(7, 58);


 /* STAGE 8: WORK OUT WHAT YOU HAVE TO PAY US BY 31 JANUARY 2005, OR WHAT WE HAVE TO PAY YOU */
 C[8][1] = C[7][58];
 showCboxf(8, 1);
 
 C[8][2] = NotLessThanZero( box[20][1] );
 showCboxf(8, 2);

 showCboxf( 8, 3 );	/* Increase in tax calculated by reference to earlier years (see page 20). Usually zero. */
 
 C[8][4] = C[8][1] + C[8][2] + C[8][3];
 showCboxf(8, 4);

 showCboxf( 8, 5 );	/* Decrease in tax calculated by reference to earlier years (see page 20). Usually zero. */
 
 showCboxf( 8, 6 );	/* If claiming carry-back (see page 20). Usually zero. */

 showCboxf( 8, 7 );	/* Payments or credits made to your 2003-04 payments on account */

 showCboxf( 8, 8 );	/* Any other payments or credits, not already included in boxes c8.5, c8.6, or c8.7, */
			/* made to your 2003-04 tax bill */

 C[8][9] = NotLessThanZero( C[8][4] - C[8][7] );
 showCboxf(8, 9);

 C[8][10] = 0.0;	/* Only used if you want an under 2,000 amount moved to 2006 tax-code */

 C[8][11] = NotLessThanZero( C[8][5] + C[8][6] + C[8][7] + C[8][8] + C[8][10] );
 showCboxf(8, 11);
 
 C[8][12] = C[8][4] - C[8][11];
 showCboxf(8, 12);

 if (C[8][12] > 0.0) fprintf(outf,"\nYou owe %6.2f\n", C[8][12] ); else
 if (C[8][12] < 0.0) fprintf(outf,"\nYou owed refund of %6.2f\n", -C[8][12] ); else
 fprintf(outf,"\nYou owe nothing.\n");
 

 /* STAGE 9: NOW CONSIDER WHETHER YOU HAVE TO MAKE PAYMENTS ON ACCOUNT FOR 2004-05 */

 C[9][1] = C[7][56] - (C[7][51] + C[8][10] );
 showCboxf(9, 1);

 if (C[9][1] < 500.0)
  {
   fprintf(outf,"You do not have to make payments on account for 2004-05.\n");
   C[9][4] = 0.0;
   showCboxf(9, 4);
  }
 else
  {
   C[9][2] = C[7][13] + C[7][34] + C[7][46] + C[7][49];
   showCboxf(9, 2);

   C[9][3] = 0.20 * C[9][2];
   showCboxf(9, 3);

   if (C[9][1] < C[9][3])
    {
     fprintf(outf,"You do not have to make payments on account for 2004-05.\n");
     C[9][4] = 0.0;
     showCboxf(9, 4);
    }
   else
    {
     C[9][4] = 0.50 * C[9][1];
     showCboxf(9, 4);
    }
  }

 C[9][5] = C[8][12] + C[9][4];
 showCboxf(9, 5);
 if (C[9][5] > 0.0)
  fprintf(outf,"Total payment you owe on 31 January 2005 = %6.2f\n", C[9][5]);
 else
  fprintf(outf,"You are owed = %6.2f\n", C[9][5]);

 fclose(infile);
 fclose(outf);

 Display_File(outfile);
 printf("\nResults written to file:  %s\n", outfile );
}

